#!/usr/local/bin/perl -w

=head1 NAME

wsdshelldriver.pl

=head1 SYNOPSIS

This program is a driver script which runs the wsdshell.pl program for an entire word sense disambiguation test collection that adheres to the directory structure specified below.

=head1 DESCRIPTION

Assuming that we have the data for a Word Sense Disambiguation collection in SENSEVAL2 format, this script requires that the various ambiguous words should be present in the following directory and file structure.

 BASE_DIRECTORY 
 |
 |_ _ word1
 |      | 
 |      |_ _ word1.sentence.sval2 (sentence mode output file)
 |      |_ _ word1.abstract.sval2 (abstract mode output file)
 |
 |_ _ word2
 |      | 
 |      |_ _ word2.sentence.sval2 (sentence mode output file)
 |      |_ _ word2.abstract.sval2 (abstract mode output file)
 |
 .
 .
 .

The "sentence" and "abstract" parts in the file names are derived from the NLM WSD collection (which is the collection for which WSDShell scripts were initially developed). They signify a smaller and wider context around the ambiguous word instances. At least one of these files should be present for this script to function.

IMPORTANT NOTE: During execution, this program changes into different sub-directories. Therefore any file paths passed at command line should be absolute (starting with volume - /, e.g. /home/mahesh/tokens etc.).

All the intermediate and output files are created inside the sub-directory for that particular word. The final Weka output is stored in a file with the full SENSEVAL2 input file name (word.sentence.sval2 or word.abstract.sval2) suffixed with ".wekaoutput" as the extension, e.g. adjustment.abstract.sval2.wekaoutput.

Additionally, to maintain some kind of information regarding the options used while processing the top level directory, the program creates an "options.log" file in the top level directory. This is simply the command line passed to the program.

=head1 USAGE

See 'wsdshelldriver.pl --help' for a list of options and their usage.

=head1 AUTHOR

Mahesh Joshi <joshi031@d.umn.edu>

Ted Pedersen <tpederse@d.umn.edu>

=head1 BUGS

=head1 SEE ALSO

NLM WSD Test Collection  :  http://wsd.nlm.nih.gov/

nlm2sval2  : http://www.d.umn.edu/~tpederse/tools.html

wsdshell.pl

=head1 COPYRIGHT

Copyright (C) 2005, Ted Pedersen and Mahesh Joshi

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

=cut

##############################################################################
############################## CHANGE LOG ####################################
##############################################################################

# 1. Date: 01/18/2005
#    Modified by: Mahesh Joshi
#    Comments: First version of script ready

##############################################################################
########################### END OF CHANGE LOG ################################
##############################################################################


use strict;

# Variable declarations

# Binary variables that determine the verbosity and the trace output of the program
my $verbose = 0;
my $trace = 0;

# Checks if user has requested help at command line
my $help = 0;

# The data directories on which we are operating
my $source_dir;


#######################
# SUB-ROUTINE SECTION #
#######################

#############
# showUsage #
#############

# This function prints out the usage of the program to STDOUT in a terse format
sub showUsage() {

    # Get the name of the program by which it is running currently
    my @tokens = split /\//, $0;
    my $filename = $tokens[@tokens-1];

    print "Usage: $filename SOURCE_ROOT_DIRECTORY WSDSHELL_OPTIONS\n";
    print "Type \"$filename --help\" for a list of options or \"perldoc $filename\" for details.\n";
}

############
# showHelp #
############

# This function prints out an intermediate level of usage information to STDOUT
# For further details, user is expected to read the perldoc for the program.
sub showHelp() {

    # Get the name of the program by which it is running currently
    my @tokens = split /\//, $0;
    my $filename = $tokens[@tokens-1];

    print "\n$filename: Driver program to process an entire test\n";
    print "collection for Word Sense Disambiguation (WSD) (converted\n";
    print "to SENSEVAL2 format).\n\n";

    print "Usage: $filename SOURCE_ROOT_DIRECTORY WSDSHELL_OPTIONS\n\n";

    print "The input source directory is mandatory. The output files\n";
    print "are created in each of the word directories inside the root\n";
    print "directory.\n\n";

    print "Valid options are:\n\n";
    print "--help         Prints this help message.\n\n";

    print "During processing, the first command line option is verified\n";
    print "by this program. The remaining options are passed verbatim to\n";
    print "wsdshell.pl program.\n\n";

    print "IMPORTANT NOTE: During execution, this program changes into\n";
    print "different sub-directories. Therefore any file paths passed at\n";
    print "command line should be absolute (starting with volume - /)\n";
    print "e.g. /home/mahesh/tokens etc.\n\n";


    exit(0);
}

#####################
# getCmdLineOptions #
#####################

# This function parses the command line of the program using the GetOpt::Long package. Additionally, it does some validations on the command line and does some pre-processing tasks like opening input and output files if required.

# So, this function is where the $hout1 and $hout2 get their appropriate values - either actual file handles or STDOUT to any one of them
sub getCmdLineOptions() {

    # Get the first command line option only
    my $arg = shift @ARGV;

	if (!defined $arg) {

		showUsage();
		exit(-1);
	}

    # Check if that is a request for help
    if ($arg eq "--help") {

        showHelp();
    }

    # Else we assume that it is the source directory
    $source_dir = $arg;
}

##############
# fatalError #
##############

# This function should be called when a critical error has occurred and the program cannot proceed anymore. It prints out the specified message at STDERR and exits from the program.

# The first parameter to it indicates whether or not the caller wants this function to print out the program usage.

# Any subsequent parameters are treated as message to be printed. In case of just one additional parameter, we assume that a simple print is required by the caller. In case of multiple subsequent arguments, we assume that a formatted printing is required and we invoke printf

# A newline is added at the end of the message by default
sub fatalError {

    # Check if showUsage should be printed after the error or not
    my $show_usage = shift;

    # Print out the message depending upon the number of additional parameters
    print STDERR "ERROR: ";

    if (@_ > 1) {
        printf STDERR @_;
    } else {
        print STDERR $_[0];
    }

    print STDERR "\n";

    # Print out usage if required
    if ($show_usage) {
        showUsage();
    }

    # Terminate the program, since this is a critical error
    exit(-1);
}

############
# tr_print #
############

# This function prints out the specified trace message at STDERR only when the tracing mode is on

# A single argument is treated as a plain string to be printed using print.
# Multiple arguments are treated as a request for formatted print, with the first argument being the format string.

# A newline is added at the end of the message by default
sub tr_print {

    if ($trace == 1) {

        if (@_ > 1) {
            printf STDERR @_;
        } else {
            print STDERR $_[0];
        }

        print STDERR "\n";
    }
}

############
# vb_print #
############

# This function prints out the specified verbose message at STDERR only when the verbose mode is on

# A single argument is treated as a plain string to be printed using print.
# Multiple arguments are treated as a request for formatted print, with the first argument being the format string.

# A newline is added at the end of the message by default
sub vb_print {

    if ($verbose == 1) {

        if (@_ > 1) {
            printf STDERR @_;
        } else {
            print STDERR $_[0];
        }

        print STDERR "\n";
    }
}

############
# sysCall #
############

# This function simply makes a system function call. It adds an error checking functionality and exits from the program if the system call has returned a non-zero exit status for the command that was executed.

sub sysCall {

    my $status = system(@_);

    # Error handling below taken from documentation of system call from 'perldoc perlfunc'
    unless ($status == 0) {

        if ($? == -1) {

            fatalError(0, "Failed to execute \"@_\": $!");

        } elsif ($? & 127) {

            fatalError(0, "Failed while executing \"@_\". Process died with signal %d, %s coredump",
                     ($? & 127),  ($? & 128) ? 'with' : 'without');
        } else {

            fatalError(0, "Failed while executing \"@_\". Process exited with value %d", $? >> 8);
        }
    }
}

#######################
# MAIN PROGRAM BEGINS #
#######################

# Process the command line options
getCmdLineOptions();

# Change to the source directory
chdir $source_dir;

# Write down the command line options that we will be passing to wsdshell.pl
# in a file in the top level directory
open (HOPTS, "> options.log") or fatalError("Failed to open file to write command line options.");

my $currdir = `pwd`;

print HOPTS "Test directory: $currdir\n";

print HOPTS "DEFAULT OPTIONS\n";
print HOPTS "===============\n";
print HOPTS "* Default statistic library - Log Likelihood measure (except when NGRAM = 1, when statistic.pl is skipped)\n";
print HOPTS "* Default NGRAM 2\n";
print HOPTS "* Default Weka classifier: weka.classifier.functions.SMO\n";
print HOPTS "========================================================\n\n";

print HOPTS "Following options were used in this test case (Defaults apply where not overridden):\n";
print HOPTS "====================================================================================\n";

my $index;

for ($index = 0; $index < @ARGV; $index++) {

    my $opt = $ARGV[$index];

    if ($opt eq "--token") {

        print HOPTS "TOKENS FILE: $ARGV[$index+1]. (Passed as option to preprocess.pl, count.pl and nsp2regex.pl)\n";

    } elsif ($opt eq "--nontoken") {

        print HOPTS "NON TOKENS FILE: $ARGV[$index+1]. (Passed as option to preprocess.pl and count.pl)\n";
    
    } elsif ($opt eq "--stop") {

        print HOPTS "STOP WORDS FILE: $ARGV[$index+1]. (Passed as option to count.pl program)\n";
    
    } elsif ($opt eq "--prepopts") {

        print HOPTS "PREPROCESS.PL OPTIONS: $ARGV[$index+1]\n";
    
    } elsif ($opt eq "--statopts") {

        print HOPTS "STATISTIC.PL OPTIONS: $ARGV[$index+1]\n";
    
    } elsif ($opt eq "--countopts") {

        print HOPTS "COUNT.PL OPTIONS: $ARGV[$index+1]\n";

    } elsif ($opt eq "--wekaopts") {

        print HOPTS "WEKA OPTIONS: $ARGV[$index+1]\n";

    } elsif ($opt eq "--wekaclassifier") {

        print HOPTS "WEKA CLASSIFIER: $ARGV[$index+1]\n";
    
    } elsif ($opt eq "--ngram") {

        print HOPTS "NGRAM: $ARGV[$index+1]\n";

    } elsif ($opt eq "--statlib") {

        print HOPTS "STATISTIC LIBRARY used for statistic.pl: $ARGV[$index+1]\n";
    }
}

print HOPTS "=============================================================\n\n\n";
close HOPTS;

# We need to enlist all the directories in the source directory
opendir(HSRCDIR, ".") or fatalError("Failed to open the directory $source_dir for reading.");

my $word;

while (($word = readdir(HSRCDIR))) {

    # We read each of the sub-directories from the source directory
    # Check that we are not dealing with the . and .. directories
    if ($word !~ /^\..*/ && -d $word) {

        print STDERR "Processing directory $source_dir/$word\n";
        
        # Change to the word directory
        chdir $word;

        # Open the word directory
        if (!opendir(HWORDDIR, ".")) {

            print STDERR "WARNING: Failed to open directory $source_dir/$word for reading. Continuing with next word.\n";
            next;
        }

        # Now we are interested in the data file inside the word directory
        my $datafile;

        # Read the directory till we get the first file
        while (($datafile = readdir(HWORDDIR))) {

            if ($datafile !~ /^\..*/ && -f $datafile && ($datafile =~ /.*\.abstract\.sval2$/ || $datafile =~ /.*\.sentence\.sval2$/)) {

                print STDERR "Processing file $datafile...\n";

                # Call the wsdshell.pl program for each data file of the word
                sysCall("wsdshell.pl", @ARGV, $datafile, "$datafile.wekaoutput");
            }
        }

        closedir HWORDDIR;

        # Change back to the parent directory
        chdir "..";
    }
}

closedir HSRCDIR;
